import smtplib
from email.mime.text import MIMEText
import config
import xml.etree.ElementTree as ET
import urllib
import urllib2
import logging

def send_email(recipient, subject, body):
    FROM = 'nagiosna-servbot@localhost'
    msg = MIMEText(body)
    msg['Subject'] = subject
    msg['From'] = FROM
    msg['To'] = recipient
    
    s = smtplib.SMTP('localhost')
    s.sendmail(FROM, [recipient], msg.as_string())
    s.quit()

def send_traps(sourcename, checkid, returncode, stdout):
    QUERY = '''
SELECT 
    `tid`
FROM
    nagiosna_ChecksTrapsLinker
WHERE
    `cid` = '%s';
''' % checkid
    devices = config.query_database(QUERY)
    
    SNMP_TMPL = '''
SELECT
    *
FROM 
    nagiosna_TrapReceivers
WHERE
    `tid` = '%s';'''
    
    for device in devices:
        info = config.query_database(SNMP_TMPL % device)
        
        try:
            d = info[0]
        except IndexError:
            logging.error('No such device: tid = %s' % device)
            continue
        
        snmp = build_snmp_trap_cmd(d, sourcename, returncode, stdout)
        print snmp
        
def build_snmp_trap_cmd(snmp_info, sourcename, returncode, stdout):
    
    cmd = ['/usr/bin/snmptrap']
    
    (   tid, 
        name, 
        version, 
        community, 
        authlevel, 
        authprotocol, 
        authpassword, 
        privprotocol, 
        privpassword, 
        username, 
        port, 
        ip              ) = snmp_info
    
    if version == 2:
        cmd += ['-v2c']
        cmd += ['-c', community]
        
    else:
        cmd += ['-v3']
        cmd += ['-u', username]
        if authlevel == 'authPriv':
            cmd += ['-a', authprotocol]
            cmd += ['-A', authpassword]
            cmd += ['-x', privprotocol]
            cmd += ['-X', privpassword]
        if authlevel == 'authNoPriv':
            cmd += ['-a', authprotocol]
            cmd += ['-A', authpassword]
    
    cmd += ['%s:%s' % (ip, port)]
    
    cmd += ['NAGIOS-NOTIFY-MIB::nSvcEvent', 'nSvcHostname', 's', 'NagiosNA']
    cmd += ['nSvcDesc', 's', str(sourcename)] 
    cmd += ['nSvcStateID', 'i', str(returncode)]
    cmd += ['nSvcOutput', 's', str(stdout)]
    
    return cmd

def get_user_emails(checkid):
    QUERY = '''
SELECT 
    `email`, `first_name`, `last_name`
FROM 
    nagiosna_users 
JOIN 
    nagiosna_ChecksUsersLinker 
ON 
    nagiosna_ChecksUsersLinker.uid = id 
WHERE 
    nagiosna_ChecksUsersLinker.cid = '%s';''' % str(checkid)
    results = config.query_database(QUERY)
    return results

def get_nrdp_info(checkid):
    QUERY = '''
SELECT 
    `hostname`, `servicename`, `nrdp`, `token` 
FROM 
    nagiosna_ChecksHSALinker hsal 
JOIN 
    nagiosna_HostServiceAssociations hsa 
ON 
    hsal.aid = hsa.id 
JOIN 
    nagiosna_NagiosServers ns 
ON 
    hsa.serverid = ns.id
WHERE
    hsal.cid = '%s';''' % str(checkid)
    results = config.query_database(QUERY)
    return results
    
def make_nrdp_xml(hostname, servicename, returncode, stdout):
    checkresults = ET.Element('checkresults')
    checkresult = ET.SubElement(checkresults, 'checkresult', type='service')
    xhostname = ET.SubElement(checkresult, 'hostname')
    xhostname.text = hostname
    xservicename = ET.SubElement(checkresult, 'servicename')
    xservicename.text = servicename
    state = ET.SubElement(checkresult, 'state')
    state.text = str(returncode)
    output = ET.SubElement(checkresult, 'output')
    output.text = stdout
    return ET.tostring(checkresults)

def post_xml_to_nrdp(url, xml, token):
    values = {  'XMLDATA': xml,
                'token': token,
                'cmd': 'submitcheck' }
    data = urllib.urlencode(values)
    req = urllib2.Request(url, data)

    print data

    try: response = urllib2.urlopen(req)
    except urllib2.HTTPError as e:
        print e.code
        print e.read()

    print response.read()
    

def send_nrdp_notifications(checkid, stdout, returncode):
    nrdp_info = get_nrdp_info(checkid)
    for hostname, servicename, nrdp, token in nrdp_info:
        xml = make_nrdp_xml(hostname, servicename, returncode, stdout)
        post_xml_to_nrdp(nrdp, xml, token)
        #~ success = send_nrdp_xml(xml)

def send_emails(sourcename, checkid, metric, rawquery, returncode, stdout):
    contact_info = get_user_emails(checkid)
    body = '''
%%s,

The source %s has been evaluated for flows given these contraints:

%s

Looking at the number of %s that passed through, here is the plugin
result:

%s

Yours Truly,

NagiosNA Servbot''' % (sourcename, rawquery, metric, stdout)
    subject = '%s on %s' % (metric, sourcename)
    for email, firstname, lastname in contact_info:
        name = "%s %s" % (firstname, lastname)
        specific = body % name
        send_email(email, subject, specific)

def send_notifications(sourcename, checkid, metric, rawquery, returncode, stdout):
    send_emails(sourcename, checkid, metric, rawquery, returncode, stdout)
    send_nrdp_notifications(checkid, stdout, returncode)
    send_traps(sourcename, checkid, returncode, stdout);
